%+---------------------------------------------------------------------------------+
%|                                                                                 |
%|                                 MATLAB SCRIPT FILE:                             |
%|                                 ===================                             |
%|                            BATCH PROCESSING OF STD DATA                         |
%|                            ============================                         |
%|                                                                                 |
%|                            Programmer.......:  V. Dirita                        |
%|                            Last Revision....: 7th Dec.92                        |
%|                                                                                 |
%| DESCRIPTION: Batch processing of STD data. Able to restore matrices  with  data |
%|              samples missing (only one sample) per cast by linear interpolation |
%|              of the above and below samples.                                    |
%|              Casts corrupted by more than one missing contiguous sample is thus |
%|              rejected.                                                          |
%|                                                                                 |
%|              Secondly,  the user provides standard depths,  which the  software |
%|              scans each file named "nodc???.dat.Z" containing high resolution   |
%|              data for one cast only: depth,temp,salt,oxyg,qual. and applies an  |
%|              averaging method to obtain values at standard depths.              |
%|                                                                                 |
%|              Thirdly, high resolution data is available to allow more  accurate |
%|              calculation of the above parameters.                               |
%|                                                                                 |          
%| FUNCTIONS..: -MissingData2():    looks for missing data in a given cast (row)   |
%|              -MissingGaps():     looks for non-uniformly spaced depth field data|
%|              -InterpolateHigh(): high resolution averaging interpolation        |
%|                                                                                 |
%+---------------------------------------------------------------------------------+


goodindex=0;
badindex=0;

for i=1:109,

   %--------------- uncompress filename: (nodcXXX.dat.Z  --> temp.dat) ------------------

   filename=sprintf('nodc%3g.dat.Z', i);
   if (filename(5)==' ') filename(5)='0'; end
   if (filename(6)==' ') filename(6)='0'; end
   if (filename(7)==' ') filename(7)='0'; end
   str=['\n\nFILE: -',filename, ' \n'];
   fprintf(str);
   fprintf('      -expanding to > temp.dat \n');
   str=['!zcat indian/', filename, ' > temp.dat'];
   eval(str);
  
 
   %------------------  convert  (temp.dat --> temp.mat)  matlab format -----------------

   fprintf('      -converting temp.dat > temp.mat \n')
   !dat2matformat.out
   load temp.mat;
   load nstd.38;


   %------------- process the file temp.mat:: <look for missing data/gaps>  -------------

   fprintf('      -looking for missing sample points........: ');
   flag_temp1=missingdata2(TEMP);
   flag_salt1=missingdata2(SALT);
   flag_oxyg1=missingdata2(OXYG);
   flag_pres1=missingdata2(PRES);
   fprintf(' \n');

   fprintf('      -looking for large gaps between samples...: ');
   maxgap=300; 
   flag_pres2=missinggaps(PRES, maxgap);
   fprintf('\n');

   fprintf('      -looking for non-monotonicity in depth ');
   flag_pres3=monotonicity(PRES);
   fprintf('\n');


   %-------------- process the file temp.mat:: <retain only good casts> -----------------

   flags=~flag_temp1 & ~flag_salt1 & ~flag_oxyg1 & ~flag_pres1 & ~flag_pres2 & ~flag_pres3;

   if (flags==1)
      fprintf('      -data ok, copying header to output file \n');
      goodindex=goodindex+1; 
      aver(goodindex)=AVER;
      bot_depth(goodindex)=BOT_DEPTH;
      cruise(goodindex)=CRUISE;
      day(goodindex)=DAY;
      inst(goodindex)=INST;
      lat(goodindex)=LAT;
      long(goodindex)=LONG;
      max_prs(goodindex)=MAX_PRS;
      month(goodindex)=MONTH;
      rate(goodindex)=RATE;
      stat(goodindex)=STAT;
      time(goodindex)=TIME;
      year(goodindex)=YEAR; 

      fprintf('      -interpolating temperature, salinity, oxygen \n');
      !interpolatehigh.out
      load temp.dat
      pressure(:,goodindex)=temp(:,1);
      temperature(:,goodindex)=temp(:,2);
      salinity(:,goodindex)=temp(:,3);
      oxygen(:,goodindex)=temp(:,4);
      quality(:,goodindex)=temp(:,5);

      N1=rowsize(TEMP);
      N2=rowsize(temperature(:,goodindex));
      str=['FILE: ', filename];

      fprintf('      -plotting graph:: temperature \n');
      plot(TEMP(1:N1), PRES(1:N1),'r', temperature(1:N2,goodindex), pressure(1:N2,goodindex), 'g')
      title(str);
      xlabel('Temperature C.');
      ylabel('Depth m.');
      grid;
      pause(1);

      fprintf('      -plotting graph:: salinity \n');
      plot(SALT(1:N1), PRES(1:N1),'r', salinity(1:N2,goodindex), pressure(1:N2,goodindex), 'g');
      title(str);
      xlabel('salinity PPT');
      ylabel('Depth m.');
      grid;
      pause(1);

      fprintf('      -plotting graph:: oxygen \n');
      plot(OXYG(1:N1), PRES(1:N1),'r', oxygen(1:N2,goodindex), pressure(1:N2,goodindex), 'g');
      title(str);
      xlabel('oxygen');
      ylabel('Depth m.');
      grid;
      pause(1);
   else 
      if (flag_temp1==1) fprintf('       ?Error::missing data in temperature column \n'); end
      if (flag_salt1==1) fprintf('       ?Error::missing data in salinity column    \n'); end
      if (flag_oxyg1==1) fprintf('       ?Error::missing data in oxygen column      \n'); end
      if (flag_pres1==1) fprintf('       ?Error::missing data in pressure column    \n'); end
      if (flag_pres2==1) fprintf('       ?Error::exceeding maximum gap in depth column  \n'); end
      if (flag_pres3==1) fprintf('       ?Error::non-monotonicity found in depth column \n'); end

      badindex=badindex+1;
      badfiles(badindex)=i;
   end
end


%------------------------------- summary of processing ------------------------------------

fprintf('\n\n BAD FILES FOUND: \n');

for j=1:badindex,
   filename=sprintf('     - nodc%3g.dat.Z', badfiles(j));
   if (filename(12)==' ') filename(12)='0'; end
   if (filename(13)==' ') filename(13)='0'; end
   if (filename(14)==' ') filename(14)='0'; end
   fprintf(filename);
end

fprintf('\n TOTAL NUMBER OF FILES PROCESSED......: %g', i);
fprintf('\n TOTAL NUMBER OF VALID RECORDS FOUND..: %g', goodindex);


%-------------------------- store results to output file nodc.mat -------------------------

fprintf('\n saving to output file nodc.mat ');
save nodc.mat aver bot_depth cruise day inst lat long max_prs month rate stat time year pressure temperature salinity oxygen quality;
fprintf('\n\n Good bye, have a nice day.... \n');
 


