function [sns,tptns,pns] = nsurf(s,tpt,p,gamma,glevels,potemp)

%--------------------------------------------------------------------
% function [sns,tptns,pns] = nsurf(s,tpt,p,gamma,glevels,potemp)
%
%          Returns (S,T,p) on neutral surfaces specified by their
%          gamma (neutral density) values.
%
% INPUT:   s       : matrix of salinities (each column must be a cast)
%          tpt     : matrix of temperatures or potential temperatures
%          p       : matrix of depths (or pressures in dbar)
%          gamma   : matrix of cast gamma values
%          glevels : vector of gamma values at which (S,T,p) required
%          potemp  = 0 if tpt is temperature
%                  = 1 if tpt is potential temperature
%
%          NOTE: Missing (S,T,p) values must be denoted by NaN's
%
% OUTPUT:  sns     : matrix of salinities on neutral surfaces
%          tptns   : matrix of temperatures or potential temperatures
%                    (same quantity as input)
%          pns     : matrix of pressures
%
%                    NaN => missing input data or no surface
%
% CALLS: sw_temp, sw_ptmp, change (all from csirolib Matlab library)
%        /home/mcintosh/fram/nsurfs/matsurf
%--------------------------------------------------------------------

% Written by Peter McIntosh
% Modified: N.L. Bindoff 16 August 1994
%
% Interface to Dave Jackett's Fortran subroutine "neutral_surfaces"

if nargin~=6
  error('Wrong number of input arguments')
end

%...Check that pressure array is monotonic increasing...
%   (This helps check if casts are in columns as required)

if length(find(diff(p)<=0)) > 0,
  error('Pressure array not monotonic increasing')
end

%...Convert potential temperature to temperature if necessary...

if potemp==1
  t=sw_temp(s,tpt,p,0.0);
elseif potemp==0
  t=tpt;
else
  error('Invalid value for potemp flag');
end

[ndep,ncast]=size(s);
g=nan*zeros(size(s));

%...Calculate maximum number of common data in casts...

index=nan*zeros(ndep,ncast);
for cast=1:ncast
  ii=find(finite(s(:,cast)+t(:,cast)+p(:,cast)+gamma(:,cast)));
  index(1:length(ii),cast)=ii;
end

%...Construct arrays in correct form for saving...

savearray=[];
for cast=1:ncast
  n=length(find(finite(index(:,cast))));
  savearray=[savearray;...
             n 0 0 0;...
             s(index(1:n,cast),cast) t(index(1:n,cast),cast)...
	     p(index(1:n,cast),cast) gamma(index(1:n,cast),cast)];
end

nlevels=length(glevels);
savelevels=[nlevels;glevels(:)];

save mat_tmp_l.dat savelevels /ascii
save mat_tmp_d.dat savearray /ascii

%...Run FORTRAN program...

%[status,output]=unix('/home/mcintosh/fram/nsurfs/matsurf');
[status,output]=unix('/u/bindoff/nsurfs/matsurf');

if status~=0,
  disp(['Error in matsurf: status = ',num2str(status)]);
  output
end

[allstp,count,err]=sscanf(output,'%f');

allstp=reshape(allstp,3,length(allstp)/3)';

!rm mat_tmp_d.dat
!rm mat_tmp_l.dat

sns=reshape(allstp(:,1),nlevels,ncast);
tns=reshape(allstp(:,2),nlevels,ncast);
pns=reshape(allstp(:,3),nlevels,ncast);

sns=change(sns,'<',-98,nan);
tns=change(tns,'<',-98,nan);
pns=change(pns,'<',-98,nan);

%...Convert temperature to potential temperature if necessary...

if potemp==1
  tptns=sw_ptmp(sns,tns,pns,0.0);
elseif potemp==0
  tptns=tns;
else
  error('Invalid value for potemp flag');
end



