function [psinp1]=vorticity2(psinm1,psin,curl,mask, ...
			delt,delx,f,c,beta,dissipation)
%**************************************************************************
%
% VORTICITY1.M
%
% programmer:	Nathan Bindoff
%		(rossby3.m modified by Neil Holbrook on 19.1.94)
% date:		December 1993
%
% modified:   Nathan Bindoff
% date:       10 March 1996
%
% subject: This function is called to numerically solve the linear
% vorticity equation along a constant latitude (i.e. ignoring the
% Laplacian term).  The numerical procedure uses a "Two-step Lax-Wendroff"
% finite difference scheme.  cf pages 623-635 Numerical Recipies (Fortran)
% (1986).  Particularly see equation 17.1.36 (p633).  The scheme also 
% incorporates a diffusion term (see equation 17.2.4 (p636)) to dissipate
% and help numerical stability.
% 
% The modifications of this routine are to make the routine work globally
% rather than along a single latitude line.  The modifications therefore 
% amount to a genralisation of the orginal vorticity1.m.
%
%**************************************************************************
%
% function [psinp1]=vorticity2(psinm1,psin,curl,mask, ...
%			delt,delx,f,c,beta,dissipation)
%
% A function designed to solve for a rossby wave
%
% Inputs:
% psinm1  -m*p matrix of streamfunction at time n-1
% psin    -m*p matrix of streamfunction at time n
% curl    -m*p matrix of wind stress curl (vertical component)
% mask    -m*p matrix of mask of land values
% delt    -scalar of time step (seconds)
% delx    -m*1 of distance along line of constant latitude (metre)
% 
% f       -m*1 vector of coriolis parameter (.s-1)
% c       -m*1 velocity of equivalent depth (m.s-1)
% beta    -m*1 vector beta .s-1
% dissipation - currently unused
%
% Output:
% psinp1  -m*p matrix of streamfunction at time n+1
%
% Notes: the variable mask consists of zeros and 1.  Zeroes indicate
% land values.
%
%**************************************************************************
%
a=(delt./delx).*(c.^2./f.^2).*beta;
b=delt*c.^2./f.^2;
cc=dissipation*delt*c.^2./f.^2./delx.^2;
%
% Create the wrap around at the edges of the global model
% 
[m,p]=size(psin);
%tmppsin=[psin(:,p),psin, psin(:,1)];
%tmpcurl=[curl(:,p),curl,curl(:,1)];
%
alpha=-a(:,ones(1,p-2));
b=b(:,ones(1,p-2));
cc=cc(:,ones(1,p-2));
%
%tmppsin(1,p+1)=-(2*b/alpha*(1+alpha))*tmpcurl(1,p+2);
%
% for the centre proportion of the model, ie where
% there are fields values
%
% keyboard
psinp1=zeros(m,p);
jj=[2:p-1];
one=ones(1,length(jj));
jjp1=jj+one;
jjm1=jj-one;
%keyboard
psinp1(:,jj)=psin(:,jj)-alpha.*(0.5*(psin(:,jjp1)+psin(:,jj)) ...
	-0.5*alpha.*(psin(:,jjp1)-psin(:,jj)) ...
	-0.5*(psin(:,jj)+psin(:,jjm1)) ...
	+0.5*alpha.*(psin(:,jj)-psin(:,jjm1))) ...
	-b.*curl(:,jj);
%
% NOTES FROM NATHAN.
% The above form of the equation is complete (i.e. without the Laplacian)
% -(f^2/c^2)*dpsi/dt+beta*dpsi/dx=curl_z(tau)
% The BELOW added term is a device to improve the stability of the
% numerical integration when using poor time-stepping schemes. The size
% of the dissipation is small and not sigificant compared with the
% other terms.  With the two step Law Wendroff scheme (described in 
% section 17.1 NUMERICAL RECIPES pages 623-635, and especially p 631)
% the dissipation term can be set to almost zero.  This term is not
% physics, but is there as a relic.  If you make it large (cf the other
% terms) you will find that the problem is a diffusion eqaution and quite
% a realistic solution, though totally irrelevant to oceanography!
psinp1(:,jj)=psinp1(:,jj)...
    +cc.*(psin(:,jjp1)-2*psin(:,jj)+psin(:,jjm1));
%
% take 
psinp1=psinp1.*mask;
return
%
%****************************************************************************
